function [Yz, Ind, Dist] = IntraKNN(X, k, p)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function is used  to find the intraclass K nearest neighbor for  %
% each sample based on different distance measure.                      %
% Neighbor can be repeated/non-repeated in this implmentation.          %
% Input: X is the input matrix for one class                            %
%        k: the number of nearest neighbors                             %
%        distance: Several distance definitions same as Pdist functions %                              %
% Output: Y is the difference matrix and Ind is an array structure for  %
% neighbor structure.                                                   %
% Zhenqiu Liu                                
% University of Maryland at Baltimore.                                  %
% zliu@umm.edu   08/19/2009                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin < 3,
   p = 1;
end;

[m, n] = size(X);
Y = pdist(X, @mypdist, p);
z = squareform(Y);
Ind = [];
Yz = [];
Dist = [];

for i =1:m,
     [nbdist, nbindex] = sort(z(i,:), 'ascend');
     knb = nbindex(2:k+1);                        % since the distance is zero for itself (k+1)
     kdist = nbdist(2:k+1); 
     ynb = repmat(X(i,:), k, 1) - X(knb', :);
     Ind = [Ind; knb];
     Yz = [Yz; ynb];      
     Dist = [Dist; kdist];
end;
end

function d = mypdist(XI, XJ, p) 
% selfdefined distance
       XIJ= abs((repmat(XI,size(XJ,1),1)-XJ)).^p ;
       d = sum(XIJ, 2);
end