function [Fv, F1] = tlsq(w, Z1, Z2, lam)
% This function calculated the Functional values and First order 
%derivative for the intra-product and inner-product distance matrix
% Z1: the inner distance components 
% Z2: the intra-distance components
%lam: the penalized term
% Fv: the functional value for truncated least square
% F1: The function for the first order derivative.
% Written by Zhenqiu Liu, University if Maryland at Baltimore
%  10/18/2010

[m1, n1] = size(Z1);
[m2, n2] = size(Z2);
y1 = ones(m1,1);
y2 = 2*ones(m2,1);
L1 = Z1*w -y1;
I1 = (L1 >0);
E1 = 0.5*sum((L1.*I1).^2);
L2 = y2-Z2*w;
I2 = (L2 >0);
E2 = 0.5*sum((L2.*I2).^2);
Fv = E1 + E2 + lam*sum(w);

X1 = Z1.* repmat(I1, 1, n1);
X2 = Z2.* repmat(I2, 1, n2);

F1 =X1'*L1 - X2'*L2 + lam;
end






